<?php
namespace Macro;
/**
 * TwitterCard - macro class
 * 
 * With Twitter Cards, you can attach rich photos, videos and media experiences to Tweets, helping to drive traffic to your website. Twitter Card is displayed on the header of the page. More info: https://developer.twitter.com/en/docs/tweets/optimize-with-cards/overview/abouts-cards
 * 
 * @copyright 2016 SCHLIX Web Inc
 *
 * @license MIT
 *
 * @package twittercard
 * @version 1.0
 * @author  SCHLIX Web Inc <info@schlix.com>
 * @link    http://www.schlix.com
 */

class twittercard extends \SCHLIX\cmsMacro {
    
    /**
     * Key/value array
     * @var array 
     */
    protected $tw_card = [];

    /**
     * Indicator to find out if the macro has been called
     * @var string 
     */
    protected static $has_this_macro_been_called;    
    
    /*
     * Run the macro
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @param array|string $data
     * @param object $caller_object
     * @param string $caller_function
     * @param array $extra_info
     * @return bool
     */   
    public function Run(&$data, $caller_object, $caller_function, $extra_info = NULL) {
        if (static::$has_this_macro_been_called != 'yes') {
            
            // only execute if it's called by an application, not block
            if (is_a($caller_object, '\\SCHLIX\\cmsApplication_Basic')) {
                switch ($caller_function) {
                    case 'viewItemByID':
                    case 'viewCategoryByID':
                    case 'viewMainPage':
                        $this->generateMeta($data, $caller_object, $caller_function);
                        break;
                    // ignore these three
                    case 'viewChildCategory': // do nothing
                    case 'viewChildItem': // do nothing
                    case 'viewMainPageItem':
                    default: break;
                }
                return true;
            }
            static::$has_this_macro_been_called = 'yes';
        }
        return false; // return false if it's a block
    }
    
    /**
     * Render twitter card from $this->tw_card
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     */
    private function renderTwitterCard()
    {
        global $HTMLHeader;
        
        if ($this->tw_card)
        {
            $HTMLHeader->Add("<!-- Twitter Card begins -->"); // optional, for test/debug
            foreach ($this->tw_card as $key => $value)
            {
                $sanitized_value = ___h($value); 
                $HTMLHeader->Add("<meta name=\"twitter:{$key}\" content=\"{$sanitized_value}\" />");
            }
            $HTMLHeader->Add("<!-- Twitter Card ends -->"); // optional, for test/debug
        }
                
    }
    
    private function indicateError()
    {
        global $HTMLHeader;
        
        $HTMLHeader->Add("<!-- Twitter card has not been set up properly. Please click Configuration -->");
    }
    /**
     * Set the twitter card variable. Don't set the key variable if it's empty
     * @param string $key
     * @param string $value
     */
    private function setTwitterCard($key, $value)
    {
        if ($value)
            $this->tw_card[$key] = trim($value);
    }
    
    
    /**
     * Given an html string, return the first found img attributes
     * @param string $html_string
     * @return array
     */
    private function getAttributesFromFirstFoundImage($html_string)
    {
        $post_image = [];
        preg_match('/<img(.+)>/', $html_string, $imgmatch);
        if (count($imgmatch) == 2)
        {
            $imgstring = $imgmatch[1];        
            preg_match_all('/\s+?(.+)="([^"]*)"/U', $imgstring, $attrs, PREG_SET_ORDER);
            //prevent stored xss eh ~ prana        
            if ($attrs) foreach ($attrs as $attr) $post_image[$attr[1]] = strip_tags ($attr[2]);
            return $post_image;
        }
        return NULL;
    }
    
    /**
     * Returns an array containing two keys: 'url' and 'alt'. A URL to a unique image representing the content of the page.
     * You should not use a generic image such as your website logo, author photo, or other image that spans multiple pages. 
     * Images for this Card support an aspect ratio of 1:1 with minimum dimensions of 144x144 or maximum of 4096x4096 pixels. 
     * Images must be less than 5MB in size. The image will be cropped to a square on all platforms. 
     * JPG, PNG, WEBP and GIF formats are supported. Only the first frame of an animated GIF will be used. SVG is not supported.
     * @param array $data
     * @param object $caller_object
     * @param string $caller_function
     * @return array
     */
    protected function getImageFromData($data, $caller_object, $caller_function)
    {
        // print_r ($data)
        $url = NULL;
        $alt = NULL;
        
        // 1. Case: Blog with field summary_intro_image
        
        if (array_key_exists("summary_intro_image", $data) && $data['summary_intro_image']) {
            $url = $data['summary_intro_image'] ? SCHLIX_SITE_URL.$caller_object->getDataFileURLPath('image_small', $data['summary_intro_image']) : '';
            $alt = $data['summary_intro_image_caption'] ? $data['summary_intro_image_caption'] : $data['title'];            
        } elseif (array_key_exists('summary',$data))
        {
            $img_attr = $this->getAttributesFromFirstFoundImage($data['summary']);
            if ($img_attr)
            {                
                $url = $img_attr['src'];
                if (!str_starts_with($url,'http://') && !str_starts_with($url,'https://'))
                        $url = SCHLIX_SITE_URL.$url;
                $alt = $img_attr['alt'];
            }
            // do your regex here. 
            // Please submit your code, I need to know that you can do this.
            // Using Google/StackOverflow is allowed
            // e.g. https://stackoverflow.com/questions/18664997/how-can-i-use-regular-expression-to-grab-an-img-tag
        }
        
        return ['url' => $url, 'alt' => $alt];
    }
    /**
     * Generate meta
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @param array $data
     * @param object $caller_object
     * @param string $caller_function
     */
    private function generateMeta($data, $caller_object, $caller_function) {
        
        $image = NULL;
        $site_author = $this->config['str_site'];
        $card_type = $this->config['str_card_type'];
        $default_title = $this->config['str_default_title'];
        $default_description = $this->config['str_default_description'];
        $title = $description = '';
        // validation
        if (empty($card_type) || empty ($site_author) || empty($default_title))
        {
            $this->indicateError();
            return; // no card type specified
        }
        // now run it
        switch ($caller_function)
        {
            case 'viewItemByID':                
            case 'viewCategoryByID':
                
                $title = $data['title'];
                //$description = $data['meta_description'] ?  $data['meta_description'] : $data['summary'];
                $description =  $data['meta_description'] ? $data['meta_description'] : $data['summary'];
                break;
            case 'viewMainPage':
                $title = $caller_object->getApplicationDescription();
                break;
                
        }
        $final_description = $description ? $description : $default_description;
        $final_description = trim(str_replace(['&nbsp;',"\n","\t", ' '],' ', strip_tags($final_description)));        
        $final_title = $title ? $title : $default_title;
        
        /** Summary
        <meta name="twitter:card" content="summary" />
        <meta name="twitter:site" content="@flickr" />
        <meta name="twitter:title" content="Small Island Developing States Photo Submission" />
        <meta name="twitter:description" content="View the album on Flickr." />
        <meta name="twitter:image" content="https://farm6.staticflickr.com/5510/14338202952_93595258ff_z.jpg" />         
        */
        
        $this->setTwitterCard('card', $card_type);
        $this->setTwitterCard('creator', $this->config['str_creator']); // if empty then it won't be set
        $this->setTwitterCard('site', $site_author);
        $this->setTwitterCard('title', str_limit($final_title, 70));
        
        $this->setTwitterCard('description', str_limit(strip_tags($final_description), 200));
        $image = $this->getImageFromData($data, $caller_object, $caller_function);
        
        if (($image != NULL) && !empty($image['url']))
        {
            $this->setTwitterCard('image', $image['url']);
            // A URL to a unique image representing the content of the page. You should not use a generic image such as your website logo, author photo, or other image that spans multiple pages. Images for this Card support an aspect ratio of 1:1 with minimum dimensions of 144x144 or maximum of 4096x4096 pixels. Images must be less than 5MB in size. The image will be cropped to a square on all platforms. JPG, PNG, WEBP and GIF formats are supported. Only the first frame of an animated GIF will be used. SVG is not supported.
            $this->setTwitterCard('image:alt', $image['alt']);
        }
        
        /** Summary with Large Image
        <meta name="twitter:card" content="summary_large_image">
        <meta name="twitter:site" content="@nytimes">
        <meta name="twitter:creator" content="@SarahMaslinNir">
        <meta name="twitter:title" content="Parade of Fans for Houston’s Funeral">
        <meta name="twitter:description" content="NEWARK - The guest list and parade of limousines with celebrities emerging from them seemed more suited to a red carpet event in Hollywood or New York than than a gritty stretch of Sussex Avenue near the former site of the James M. Baxter Terrace public housing project here.">
        <meta name="twitter:image" content="http://graphics8.nytimes.com/images/2012/02/19/us/19whitney-span/19whitney-span-articleLarge.jpg">
         */
        // no change
        if ($card_type == 'app')
        {
            $this->setTwitterCard('app:country',  $this->config['str_app_country']);
            $this->setTwitterCard('app:name:iphone',  $this->config['str_iphone_name']);
            $this->setTwitterCard('app:id:iphone',  $this->config['str_iphone_id']);
            $this->setTwitterCard('app:url:iphone',  $this->config['str_iphone_url']);
            $this->setTwitterCard('app:name:ipad',  $this->config['str_ipad_name']);
            $this->setTwitterCard('app:id:ipad',  $this->config['str_ipad_id']);
            $this->setTwitterCard('app:url:ipad',  $this->config['str_ipad_url']);
            $this->setTwitterCard('app:name:googleplay',  $this->config['str_gplay_name']);
            $this->setTwitterCard('app:id:googleplay',  $this->config['str_gplay_url']);
            $this->setTwitterCard('app:url:googleplay',  $this->config['str_gplay_id']);
            
        }
        /** App Card sample
        <meta name="twitter:card" content="app">
        <meta name="twitter:site" content="@TwitterDev">
        <meta name="twitter:description" content="Cannonball is the fun way to create and share stories and poems on your phone. Start with a beautiful image from the gallery, then choose words to complete the story and share it with friends.">
        <meta name="twitter:app:country" content="US">
        <meta name="twitter:app:name:iphone" content="Cannonball">
        <meta name="twitter:app:id:iphone" content="929750075">
        <meta name="twitter:app:url:iphone" content="cannonball://poem/5149e249222f9e600a7540ef">
        <meta name="twitter:app:name:ipad" content="Cannonball">
        <meta name="twitter:app:id:ipad" content="929750075">
        <meta name="twitter:app:url:ipad" content="cannonball://poem/5149e249222f9e600a7540ef">
        <meta name="twitter:app:name:googleplay" content="Cannonball">
        <meta name="twitter:app:id:googleplay" content="io.fabric.samples.cannonball">
        <meta name="twitter:app:url:googleplay" content="http://cannonball.fabric.io/poem/5149e249222f9e600a7540ef">
         */
        
        // when it's finally done, render the twitter card
        // it's cleaner since you don't have to call global $HTMLHeader
        // repeatedly
        $this->renderTwitterCard();
        
        
    }
    /**
     * Generate meta
     * @global \SCHLIX\cmsHTMLPageHeader $HTMLHeader
     * @param array $data
     * @param object $caller_object
     * @param string $caller_function
     */
    private function generate_meta_OLD_YH($data, $caller_object, $caller_function) {
        global $HTMLHeader;
        if (static::$has_this_macro_been_called != 'yes') {
            // filter xss
            $creator = ___h($this->config['str_creator']);

            $title = ___h($this->config['str_title']);
            if (!isset($title) || $title == "") {
                $title = ___h($data['title']);
            }
            $title = str_limit($title, 70); // strlen($title) > 70 ? substr($title, 0, 70) : $title;

            $desc = ___h($this->config['str_post_desc']);
            if (!isset($desc) || $desc == "") {
                $desc = ___h($data['meta_description']);
            }
            $desc = strlen($desc) > 200 ? substr($desc, 0, 200) : $desc;

            $card_type = $this->config['str_card_type'];

            //Set the Header property
            if (isset($creator)) {
                $HTMLHeader->Add('<meta name="twitter:creator:id" content="' . $creator . '" />');
            }
            $HTMLHeader->Add('<meta name="twitter:title" content="' . $title . '" />');
            $HTMLHeader->Add('<meta name="twitter:description" content="' . $desc . '" />');

            if (isset($card_type) && $card_type != "") {
                $HTMLHeader->Add('<meta name="twitter:card" content="' . $card_type . '" />');
            } else {
                $HTMLHeader->Add('<meta name="twitter:card" content="summary" />');
            }

            // Search for image
            if (array_key_exists("summary_intro_image", $data)) {
                $url_image = $data['summary_intro_image'] ? $caller_object->getDataFileURLPath('image_small', $data['summary_intro_image']) : '';

                if(!$url_image && array_key_exists('summary',$data)){
                    $image = preg_match("\.(?:jpg|jpeg|gif|png)", $data);
                    if($image){
                        // Image exist
                    }
                }
            }

            if ($url_image) {
                $HTMLHeader->Add('<meta property="twitter:image" content="' . SCHLIX_SITE_URL . $url_image . '" />');
            }
            if(array_key_exists("summary_intro_image_caption", $data)){
                $HTMLHeader->Add('<meta property="twitter:image:alt" content="' . ___h($data['summary_intro_image_caption']). '" />');
            }
            if ($card_type == "app") {
                $iphone_name = $this->config['str_iphone_name'];
                $iphone_url = $this->config['str_iphone_url'];
                $iphone_id = $this->config['str_iphone_id'];
                $ipad_name = $this->config['str_ipad_name'];
                $ipad_url = $this->config['str_ipad_url'];
                $ipad_id = $this->config['str_ipad_id'];
                $gplay_name = $this->config['str_gplay_name'];
                $gplay_url = $this->config['str_gplay_url'];
                $gplay_id = $this->config['str_gplay_id'];

                $HTMLHeader->Add('<meta name="twitter:app:name:iphone" content="' . $iphone_name . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:id:iphone" content="' . $iphone_id . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:url:iphone" content="' . $iphone_url . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:name:ipad" content="' . $ipad_name . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:id:ipad" content="' . $ipad_id . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:url:ipad" content="' . $ipad_url . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:name:googleplay" content="' . $gplay_name . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:id:googleplay" content="' . $gplay_id . '" />');
                $HTMLHeader->Add('<meta name="twitter:app:url:googleplay" content="' . $gplay_url . '" />');
            }
        }
    }

}